@@ -1,5 +1,6 @@
 #!/usr/bin/python3
 import argparse
+import os
 from pathlib import Path
 import shutil
 import subprocess
@@ -18,14 +19,6 @@ from tuxrun.yaml import yaml_load
 #############
 # Constants #
 #############
-ALIASES = {
-    "qemu-arm": "qemu-armv7",
-    "qemu-mips": "qemu-mips64",
-    "qemu-powerpc": "qemu-ppc64",
-    "qemu-riscv": "qemu-riscv64",
-}
-
-
 COLORS = {
     "exception": "\033[1;31m",
     "error": "\033[1;31m",
@@ -45,17 +38,29 @@ DEVICES = [
     "qemu-armv7",
     "qemu-arm64",
     "qemu-i386",
-    "qemu-mips",
     "qemu-mips64",
     "qemu-mips64el",
-    "qemu-powerpc",
     "qemu-ppc64",
-    "qemu-riscv",
+    "qemu-ppc64le",
     "qemu-riscv64",
     "qemu-sparc64",
     "qemu-x86_64",
 ]
 
+KERNELS = {
+    "qemu-armv5": "https://storage.tuxboot.com/armv5/zImage",
+    "qemu-armv7": "https://storage.tuxboot.com/armv7/zImage",
+    "qemu-arm64": "https://storage.tuxboot.com/arm64/Image",
+    "qemu-i386": "https://storage.tuxboot.com/i386/bzImage",
+    "qemu-mips64": "https://storage.tuxboot.com/mips64/vmlinux",
+    "qemu-mips64el": "https://storage.tuxboot.com/mips64el/vmlinux",
+    "qemu-ppc64": "https://storage.tuxboot.com/ppc64/vmlinux",
+    "qemu-ppc64le": "https://storage.tuxboot.com/ppc64le/vmlinux",
+    "qemu-riscv64": "https://storage.tuxboot.com/riscv64/Image",
+    "qemu-sparc64": "https://storage.tuxboot.com/sparc64/vmlinux",
+    "qemu-x86_64": "https://storage.tuxboot.com/x86_64/bzImage",
+}
+
 
 ###########
 # Helpers #
@@ -103,6 +108,10 @@ def setup_parser() -> argparse.ArgumentParser:
     group.add_argument("--kernel", default=None, type=pathurlnone, help="kernel URL")
     group.add_argument("--modules", default=None, type=pathurlnone, help="modules URL")
     group.add_argument("--rootfs", default=None, type=pathurlnone, help="rootfs URL")
+    group.add_argument(
+        "--partition", default=None, type=int, help="rootfs partition number"
+    )
+    group.add_argument("--bios", default=None, type=pathurlnone, help="bios URL")
     group.add_argument("--dtb", default=None, type=pathurlnone, help="dtb URL")
     group.add_argument(
         "--tests",
@@ -150,17 +159,26 @@ def setup_parser() -> argparse.ArgumentParser:
 def _main(options, tmpdir: Path) -> int:
     # Render the job definition and device dictionary
     if options.device:
+        kernel_compression = None
+        if options.kernel.endswith(".gz"):
+            kernel_compression = "gz"
+        if options.kernel.endswith(".xz"):
+            kernel_compression = "xz"
+
         definition = templates.jobs.get_template(
             f"{options.device}.yaml.jinja2"
         ).render(
             device=options.device,
             kernel=options.kernel,
             modules=options.modules,
+            bios=options.bios,
             dtb=options.dtb,
             rootfs=options.rootfs,
+            rootfs_partition=options.partition,
             tests=options.tests,
             timeouts=templates.timeouts(),
             tux_boot_args=options.boot_args.replace('"', ""),
+            kernel_compression=kernel_compression,
         )
         debug(options, "job definition")
         debug(options, definition)
@@ -197,12 +215,26 @@ def _main(options, tmpdir: Path) -> int:
             "/boot:/boot:ro",
             "/lib/modules:/lib/modules:ro",
         ]
-        for path in [options.kernel, options.modules, options.dtb, options.rootfs]:
+        for path in [
+            options.kernel,
+            options.modules,
+            options.bios,
+            options.dtb,
+            options.rootfs,
+        ]:
             if not path:
                 continue
             if urlparse(path).scheme == "file":
                 bindings.append(f"{path[7:]}:{path[7:]}:ro")
 
+        # Bind /dev/kvm is available
+        if Path("/dev/kvm").exists():
+            bindings.append("/dev/kvm:/dev/kvm:rw")
+        # Bind /var/tmp/.guestfs-$id if available
+        guestfs = Path(f"/var/tmp/.guestfs-{os.getuid()}")
+        if guestfs.exists():
+            bindings.append(f"{guestfs}:/var/tmp/.guestfs-0:rw")
+
         if options.runtime == "docker":
             runtime_args = ["docker", "run"]
         elif options.runtime == "podman":
@@ -284,18 +316,21 @@ def main() -> int:
         )
         return 1
 
-    # --device/--kernel are mandatory
+    # --device are mandatory
     if first_group:
         if not options.device:
             parser.print_usage()
             sys.stderr.write("tuxrun: error: argument --device is required\n")
             return 1
         if not options.kernel:
-            parser.print_usage()
-            sys.stderr.write("tuxrun: error: argument --kernel is required\n")
-            return 1
+            options.kernel = KERNELS[options.device]
 
-        options.device = ALIASES.get(options.device, options.device)
+        if options.bios and options.device != "qemu-riscv64":
+            parser.print_usage()
+            sys.stderr.write(
+                "tuxrun: error: argument --bios is only valid for qemu-riscv64 device\n"
+            )
+            return 1
 
         if options.dtb and options.device != "qemu-armv5":
             parser.print_usage()
